import { Component } from '@angular/core';
import {
  CdkDragDrop,
  CdkDropList,
  CdkDrag,
  moveItemInArray,
} from '@angular/cdk/drag-drop';
import { DetailsService } from 'src/services/details/details.service';
import { DataService } from 'src/services/data/data.service';
import { SimpleItem } from 'src/interfaces/simple-item';
import { Food } from 'src/interfaces/food';
import { SimpleItemDetailsComponent } from './simple-item-details/simple-item-details.component';
import { ModalService } from 'src/services/modal/modal.service';
import { SimpleItemModalComponent } from './simple-item-modal/simple-item-modal.component';
import { FoodDetailsComponent } from './food-details/food-details.component';

@Component({
  selector: 'app-journal-inventory',
  templateUrl: './journal-inventory.component.html',
  styleUrls: ['./journal-inventory.component.scss'],
})
export class JournalInventoryComponent {
  constructor(
    private detailsAccessor: DetailsService,
    private dataAccessor: DataService,
    private modalAccessor: ModalService
  ) {}

  public active: number = 1;

  public weaponsAndArmor: SimpleItem[] = [];
  public consumables: SimpleItem[] = [];
  public miscellaneous: SimpleItem[] = [];
  public money: any = {};
  public food: Food[] = [];

  public ngOnInit(): void {
    this.weaponsAndArmor = this.dataAccessor.items;
    this.consumables = this.dataAccessor.consumables;
    this.miscellaneous = this.dataAccessor.miscellaneous;
    this.food = this.dataAccessor.food;
    this.money = this.dataAccessor.money;
  }

  drop(event: CdkDragDrop<string[]>, list: any[], listName: string) {
    moveItemInArray(list, event.previousIndex, event.currentIndex);
    if (listName === 'weaponsAndArmor') {
      this.dataAccessor.items = list;
    } else if (listName === 'food') {
      this.dataAccessor.food = list;
    } else if (listName === 'consumables') {
      this.dataAccessor.consumables = list;
    } else if (listName === 'miscellaneous') {
      this.dataAccessor.miscellaneous = list;
    } else {
      throw new Error('Unknown list name: ' + listName);
    }
  }

  public openItemsDetails(list: any, index: number, listName: string): void {
    this.detailsAccessor.openPanel(SimpleItemDetailsComponent, {
      item: list[index],
    });
    const resultSubscription = this.detailsAccessor.result$.subscribe(
      (result) => {
        console.log(result);
        if (result.state === 'delete') {
          list.splice(index, 1);
          this.updateDatabase(listName);
        } else if (result.state === 'update') {
          this.openItemModal(list, index, listName);
        }
        resultSubscription.unsubscribe();
      }
    );
  }

  public openFoodDetails(index: number): void {
    this.detailsAccessor.openPanel(FoodDetailsComponent, {
      item: this.food[index],
    });
    const resultSubscription = this.detailsAccessor.result$.subscribe(
      (result) => {
        console.log('result kam zurück:', result);
        if (result.state === 'delete') {
          this.food.splice(index, 1);
          this.dataAccessor.food = this.food;
        } else if (result.state === 'update') {
          this.openFoodModal(index);
        }
        resultSubscription.unsubscribe();
      }
    );
  }

  public updateDatabase(listname: string): void {
    if (listname === 'weaponsAndArmor') {
      this.dataAccessor.items = this.weaponsAndArmor;
    } else if (listname === 'consumables') {
      this.dataAccessor.consumables = this.consumables;
    } else if (listname === 'miscellaneous') {
      this.dataAccessor.miscellaneous = this.miscellaneous;
    } else {
      throw new Error('Unknown list name: ' + listname);
    }
  }

  public updateMoney(): void {
    this.dataAccessor.money = this.money;
  }

  public updateFood(): void {
    this.dataAccessor.food = this.food;
  }

  public openFoodModal(index: number): void {
    this.modalAccessor.openModal(SimpleItemModalComponent, {
      item: JSON.parse(JSON.stringify(this.food[index])),
    });
    const resultSubscription = this.modalAccessor.result$.subscribe(
      (result) => {
        console.log('result kam zurück: ', result);
        if (result.state === 'update') {
          this.food[index] = result.data;
          this.updateFood();
        }
        resultSubscription.unsubscribe();
      }
    );
  }

  public openItemModal(
    list: SimpleItem[],
    index: number,
    listname: string
  ): void {
    this.modalAccessor.openModal(SimpleItemModalComponent, {
      item: JSON.parse(JSON.stringify(list[index])),
    });
    const resultSubscription = this.modalAccessor.result$.subscribe(
      (result) => {
        console.log(result);
        if (result.state === 'update') {
          list[index] = result.data;
          if (listname === 'weaponsAndArmor') {
            this.updateDatabase('weaponsAndArmor');
          } else if (listname === 'consumables') {
            this.updateDatabase('consumables');
          } else if (listname === 'miscellaneous') {
            this.updateDatabase('miscellaneous');
          } else {
            throw new Error('Unknown list name: ' + listname);
          }
        }
        resultSubscription.unsubscribe();
      }
    );
  }
}
