import { Component, Input, Output, EventEmitter } from '@angular/core';
import {
  CdkDragDrop,
  CdkDropList,
  CdkDrag,
  moveItemInArray,
} from '@angular/cdk/drag-drop';
import { NgxSmartModalService } from 'ngx-smart-modal';

@Component({
  selector: 'tools-modal',
  templateUrl: './tools-modal.component.html',
  styleUrls: ['./tools-modal.component.scss'],
})
export class ToolsModalComponent {
  public constructor(public ngxSmartModalService: NgxSmartModalService) {}

  public proficiencies: any = { tools: [], languages: [] };
  @Output() public proficienciesUpdated = new EventEmitter<any>();

  public loadData(): void {
    this.proficiencies = JSON.parse(
      JSON.stringify(
        this.ngxSmartModalService.getModalData('toolsAndLanguagesModal')
      )
    );
    this.ngxSmartModalService.resetModalData('toolsAndLanguagesModal');

    console.log('Proficiencies: ', this.proficiencies);
  }

  public dropTools(event: CdkDragDrop<string[]>): void {
    moveItemInArray(
      this.proficiencies.tools,
      event.previousIndex,
      event.currentIndex
    );
  }

  public dropLanguages(event: CdkDragDrop<string[]>): void {
    moveItemInArray(
      this.proficiencies.languages,
      event.previousIndex,
      event.currentIndex
    );
  }

  public addTool(): void {
    this.proficiencies.tools.push('');
  }

  public deleteTool(index: number): void {
    this.proficiencies.tools.splice(index, 1);
  }

  public addLanguage(): void {
    this.proficiencies.languages.push('');
  }

  public deleteLanguage(index: number): void {
    this.proficiencies.languages.splice(index, 1);
  }

  public updateProficiencies(): void {
    this.proficienciesUpdated.emit(this.proficiencies);
  }
}
