import { Component, Input, Output, EventEmitter } from '@angular/core';
import { NgxSmartModalService } from 'ngx-smart-modal';

@Component({
  selector: 'spellslots-modal',
  templateUrl: './spellslots-modal.component.html',
  styleUrls: ['./spellslots-modal.component.scss'],
})
export class SpellslotsModalComponent {
  public constructor(public ngxSmartModalService: NgxSmartModalService) {}

  @Input() public spellslots: any[] = [];
  @Input() public kiPoints: any;
  @Input() public showSpellslots: boolean = true;

  @Output() public slotsAndPointsUpdated = new EventEmitter<any>();

  public spellNumbersArray: number[] = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
  public kiNumbersArray: number[] = [
    1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  ];

  ngOnInit(): void {
    // Create a deep copy of the spellslots and kiPoints objects so they are not modified in the parent component
    this.kiPoints = JSON.parse(JSON.stringify(this.kiPoints));
    this.spellslots = JSON.parse(JSON.stringify(this.spellslots));
  }

  // Shows the spellslot container
  public onSpellslotsSwitchChanged(event: any): void {
    this.showSpellslots = event.target.checked;
  }

  // Shows the ki points container
  public onKiPointsSwitchChanged(event: any): void {
    this.kiPoints.showKiPoints = event.target.checked;
  }

  // Removes the specified spellslot level
  public removeSpellLevel(levelIndex: number): void {
    this.spellslots.splice(levelIndex, 1);
  }

  // adds a new spellslot level
  public addSpellLevel(): void {
    this.spellslots.push({
      totalSlots: 1,
      usedSlots: 0,
    });
  }

  public updateSlotsandPoints(): void {
    this.spellslots.forEach((level) => {
      if (level.usedSlots > level.totalSlots) {
        level.usedSlots = level.totalSlots;
      }
    });
    if (this.kiPoints.usedPoints > this.kiPoints.totalPoints) {
      this.kiPoints.usedPoints = this.kiPoints.totalPoints;
    }
    const updatedSlotsAndPoints = {
      spellslots: {
        spellslots: this.spellslots,
        showSpellslots: this.showSpellslots,
      },
      kiPoints: { kiPoints: this.kiPoints },
    };

    this.slotsAndPointsUpdated.emit(updatedSlotsAndPoints);
    this.ngxSmartModalService.close('spellslotsModal');
  }

  public abortUpdate(): void {
    this.ngxSmartModalService.close('spellslotsModal');
  }
}
