import { Component, Output, EventEmitter, Input } from '@angular/core';
import { NgxSmartModalService } from 'ngx-smart-modal';
import { Trait } from 'src/interfaces/traits';

@Component({
  selector: 'trait-modal',
  templateUrl: './trait-modal.component.html',
  styleUrls: ['./trait-modal.component.scss'],
})
export class TraitModalComponent {
  public constructor(public ngxSmartModalService: NgxSmartModalService) {}

  @Output() public traitCreated: EventEmitter<Trait> =
    new EventEmitter<Trait>();

  @Output() public traitUpdated: EventEmitter<any> = new EventEmitter<any>();

  @Output() public traitDelete: EventEmitter<number> =
    new EventEmitter<number>();

  @Input() public isToUpdate: boolean = false;
  @Input() public traitToUpdate: Trait | undefined;

  public newTraitName: string = '';
  public newTraitShortDescription: string = '';
  public newTraitLongDescription: string = '';
  public newTraitOrigin: string = '';

  public createTrait(): void {
    const newTrait: Trait = {
      name: this.newTraitName,
      shortDescription: this.newTraitShortDescription,
      longDescription: this.newTraitLongDescription,
      origin: this.newTraitOrigin,
    };

    this.traitCreated.emit(newTrait);
    this.ngxSmartModalService.getModal('traitModal').close();
    this.resetData();
  }

  public checkIfTraitIsToUpdate(): void {
    if (this.isToUpdate) {
      this.newTraitName = this.traitToUpdate!.name;
      this.newTraitShortDescription = this.traitToUpdate!.shortDescription;
      this.newTraitLongDescription = this.traitToUpdate!.longDescription;
      this.newTraitOrigin = this.traitToUpdate!.origin;
    }
  }

  public updateTrait(): void {
    const updatedTrait: Trait = {
      name: this.newTraitName,
      shortDescription: this.newTraitShortDescription,
      longDescription: this.newTraitLongDescription,
      origin: this.newTraitOrigin,
    };

    this.traitUpdated.emit(updatedTrait);
    this.ngxSmartModalService.getModal('traitModal').close();
    this.resetData();
  }

  public resetData(): void {
    this.newTraitName = '';
    this.newTraitShortDescription = '';
    this.newTraitLongDescription = '';
    this.newTraitOrigin = '';
  }
}
