import { Component } from '@angular/core';
import {
  CdkDragDrop,
  CdkDropList,
  CdkDrag,
  moveItemInArray,
} from '@angular/cdk/drag-drop';
import { DataService } from 'src/services/data/data.service';
import { DetailsService } from 'src/services/details/details.service';
import { NgxSmartModalService } from 'ngx-smart-modal';
import { Trait } from 'src/interfaces/traits';
import { TraitDetailsComponent } from './trait-details/trait-details.component';

@Component({
  selector: 'trait-table',
  templateUrl: './trait-table.component.html',
  styleUrls: ['./trait-table.component.scss'],
})
export class TraitTableComponent {
  public traits!: Trait[];

  public isToUpdate: boolean = false;
  public traitToUpdate: Trait | undefined;
  public updateTraitIndex: number | undefined;

  public constructor(
    public dataAccessor: DataService,
    public ngxSmartModalService: NgxSmartModalService,
    private detailsAccesssor: DetailsService
  ) {}

  public ngOnInit(): void {
    this.traits = this.dataAccessor.getTraits();
  }

  public drop(event: CdkDragDrop<string[]>): void {
    moveItemInArray(this.traits, event.previousIndex, event.currentIndex);
    this.updateDatabase();
  }

  public openTraitModal(traitIndex?: number): void {
    if (traitIndex !== undefined) {
      this.isToUpdate = true;
      this.traitToUpdate = this.traits[traitIndex];
      this.updateTraitIndex = traitIndex;
    }
    this.ngxSmartModalService.getModal('traitModal').open();
  }

  // add
  public addNewlyCreatedTrait(trait: Trait): void {
    this.traits.push(trait);
    this.updateDatabase();
  }

  //  update
  public updateTrait(ability: Trait): void {
    this.traits[this.updateTraitIndex!] = ability;
    this.updateDatabase();
    this.resetUpdateData();
  }

  public updateDatabase(): void {
    this.dataAccessor.setTraits(this.traits);
  }

  private resetUpdateData(): void {
    this.isToUpdate = false;
    this.traitToUpdate = undefined;
    this.updateTraitIndex = undefined;
  }

  private deleteTrait(index: number): void {
    this.traits.splice(index, 1);
    this.updateDatabase();
  }

  // details Panel

  public openDetailsPanel(index: number): void {
    this.detailsAccesssor.openPanel(TraitDetailsComponent, {
      trait: this.traits[index],
    });
    const resultSubscription = this.detailsAccesssor.result$.subscribe(
      (result) => {
        console.log(result);
        if (result.state === 'delete') {
          this.deleteTrait(index);
        } else if (result.state === 'update') {
          this.openTraitModal(index);
        }
        resultSubscription.unsubscribe();
      }
    );
  }
}
