import { Component, EventEmitter, Output } from '@angular/core';
import { NgxSmartModalService } from 'ngx-smart-modal';
import { Damage } from 'src/interfaces/damage';
import { Weapon } from 'src/interfaces/weapon';

@Component({
  selector: 'weapon-modal',
  templateUrl: './weapon-modal.component.html',
  styleUrls: ['./weapon-modal.component.scss'],
})
export class WeaponModalComponent {
  public constructor(public ngxSmartModalService: NgxSmartModalService) {}

  @Output() public weaponCreated: EventEmitter<Weapon> =
    new EventEmitter<Weapon>();

  @Output() public weaponUpdated: EventEmitter<any> = new EventEmitter<any>();

  @Output() public weaponDelete: EventEmitter<number> =
    new EventEmitter<number>();

  public newWeaponName: string = '';
  public newWeaponRange: string = '5 ft.';
  public newWeaponThrowRange: string = '';
  public newWeaponAttackBonus: string = '+0';
  public newWeaponDamage: Damage[] = [
    { diceNumber: '', diceType: '', damageType: '' },
  ];
  public newWeaponProficient: boolean = false;
  public newWeaponIsVersatile: boolean = false;
  public newWeaponIsTwoHanded: boolean = false;
  public newWeaponIsFinesse: boolean = false;
  public newWeaponIsRanged: boolean = false;
  public newWeaponVersatileDamage: string = '';
  public newWeaponCanBeThrown: boolean = false;
  public newWeaponWeight: string = 'normal';
  public newWeaponIsMagical: boolean = false;
  public newWeaponMagicBonus: number = 0;

  public indexForUpdate: number | undefined;
  public isUpdate: boolean = false;

  // Options for the select boxes
  public weights: string[] = ['leicht', 'normal', 'schwer'];

  public damageTypes: any[] = [
    { display: 'Wucht', value: 'bludgeoning' },
    { display: 'Stich', value: 'piercing' },
    { display: 'Hieb', value: 'slashing' },
    { display: 'Feuer', value: 'fire' },
    { display: 'Kälte', value: 'cold' },
    { display: 'Blitz', value: 'lightning' },
    { display: 'Gift', value: 'poison' },
    { display: 'Säure', value: 'acid' },
    { display: 'Nekrotisch', value: 'necrotic' },
    { display: 'Psychisch', value: 'psychic' },
    { display: 'Heilig', value: 'holy' },
    { display: 'Göttlich', value: 'divine' },
    { display: 'Kraft', value: 'force' },
  ];

  public dice: string[] = ['d4', 'd6', 'd8', 'd10', 'd12', 'd20', 'd100'];

  public numbers: string[] = [
    '1',
    '2',
    '3',
    '4',
    '5',
    '6',
    '7',
    '8',
    '9',
    '10',
  ];

  public attackBonuses: string[] = [
    '-5',
    '-4',
    '-3',
    '-2',
    '-1',
    '+0',
    '+1',
    '+2',
    '+3',
    '+4',
    '+5',
    '+6',
    '+7',
    '+8',
    '+9',
    '+10',
    '+11',
    '+12',
    '+13',
    '+14',
    '+15',
    '+16',
    '+17',
    '+18',
    '+19',
    '+20',
  ];

  public magicBonuses: any[] = [
    { display: '+1', value: 1 },
    { display: '+2', value: 2 },
    { display: '+3', value: 3 },
    { display: '+4', value: 4 },
    { display: '+5', value: 5 },
  ];
  //

  public createWeapon(): void {
    const newWeapon: Weapon = {
      name: this.newWeaponName,
      damage: this.newWeaponDamage,
      attackBonus: this.newWeaponAttackBonus,
      range: this.newWeaponRange,
      throwRange: this.newWeaponThrowRange,
      isFinesse: this.newWeaponIsFinesse,
      proficient: this.newWeaponProficient,
      isTwoHanded: this.newWeaponIsTwoHanded,
      isVersatile: this.newWeaponIsVersatile,
      isRanged: this.newWeaponIsRanged,
      versatileDamage: this.newWeaponVersatileDamage,
      canBeThrown: this.newWeaponCanBeThrown,
      weight: this.newWeaponWeight,
      isMagical: this.newWeaponIsMagical,
      magicBonus: this.newWeaponMagicBonus,
    };
    this.weaponCreated.emit(newWeapon);
    this.ngxSmartModalService.closeLatestModal();
  }

  public addDamage(): void {
    this.newWeaponDamage.push({ diceNumber: '', diceType: '', damageType: '' });
  }

  public removeDamage(index: number): void {
    this.newWeaponDamage.splice(index, 1);
  }

  public removeData(): void {
    this.newWeaponName = '';
    this.newWeaponRange = '5 ft.';
    this.newWeaponAttackBonus = '+0';
    this.newWeaponDamage = [{ diceNumber: '', diceType: '', damageType: '' }];
    this.newWeaponProficient = false;
    this.newWeaponIsVersatile = false;
    this.newWeaponIsTwoHanded = false;
    this.newWeaponIsFinesse = false;
    this.newWeaponIsRanged = false;
    this.newWeaponVersatileDamage = '';
    this.newWeaponWeight = 'normal';
    this.newWeaponIsMagical = false;
    this.newWeaponMagicBonus = 0;
    this.isUpdate = false;
    this.ngxSmartModalService.resetModalData('weaponModal');
  }

  // update

  public checkIfUpdate(): void {
    if (this.ngxSmartModalService.getModalData('weaponModal')) {
      this.isUpdate = true;
      const data: any = this.ngxSmartModalService.getModalData('weaponModal');
      const weapon = data.weapon;
      this.newWeaponName = weapon.name;
      this.newWeaponRange = weapon.range;
      this.newWeaponAttackBonus = weapon.attackBonus;
      this.newWeaponDamage = weapon.damage;
      this.newWeaponProficient = weapon.proficient;
      this.newWeaponIsVersatile = weapon.isVersatile;
      this.newWeaponIsTwoHanded = weapon.isTwoHanded;
      this.newWeaponIsFinesse = weapon.isFinesse;
      this.newWeaponIsRanged = weapon.isRanged;
      this.newWeaponVersatileDamage = weapon.versatileDamage;
      this.newWeaponWeight = weapon.weight;
      this.newWeaponIsMagical = weapon.isMagical;
      this.newWeaponMagicBonus = weapon.magicBonus;

      this.indexForUpdate = data.index;
    }
  }

  public updateWeapon(): void {
    const newWeapon: Weapon = {
      name: this.newWeaponName,
      damage: this.newWeaponDamage,
      attackBonus: this.newWeaponAttackBonus,
      range: this.newWeaponRange,
      throwRange: this.newWeaponThrowRange,
      isFinesse: this.newWeaponIsFinesse,
      proficient: this.newWeaponProficient,
      isTwoHanded: this.newWeaponIsTwoHanded,
      isVersatile: this.newWeaponIsVersatile,
      isRanged: this.newWeaponIsRanged,
      versatileDamage: this.newWeaponVersatileDamage,
      canBeThrown: this.newWeaponCanBeThrown,
      weight: this.newWeaponWeight,
      isMagical: this.newWeaponIsMagical,
      magicBonus: this.newWeaponMagicBonus,
    };
    this.weaponUpdated.emit({ weapon: newWeapon, index: this.indexForUpdate });
    this.ngxSmartModalService.closeLatestModal();
    this.removeData();
  }

  public deleteWeapon(): void {
    this.weaponDelete.emit(this.indexForUpdate);
    this.ngxSmartModalService.closeLatestModal();
    this.removeData();
  }

  public dismiss(): void {
    this.ngxSmartModalService.closeLatestModal();
    this.removeData();
  }
}
