import { Injectable } from '@angular/core';
import Localbase from 'localbase';
import { Character } from 'src/interfaces/character';
import { Router } from '@angular/router';
import { BehaviorSubject } from 'rxjs';
import { Attribute } from 'src/interfaces/attribute';
import { Skill } from 'src/interfaces/skill';
import { Weapon } from 'src/interfaces/weapon';
import { Spell } from 'src/interfaces/spell';
import { Ability } from 'src/interfaces/ability';
import { Trait } from 'src/interfaces/traits';
import { SimpleItem } from 'src/interfaces/simple-item';
import { Food } from 'src/interfaces/food';

@Injectable({
  providedIn: 'root',
})
export class DataService {
  db: any;

  constructor(private Router: Router) {
    this.db = new Localbase('myDatabase');
    // this.test();
  }

  // public test(): void {
  //   let char: Character = {
  //     characterData: {
  //       name: 'Eliane',
  //       class: '',
  //       race: '',
  //       level: '',
  //       subclass: '',
  //       background: '',
  //       experience: '',
  //       inspiration: '',
  //       proficiencyBonus: '2',
  //     },
  //     attributes: {
  //       strength: ['12', false],
  //       dexterity: ['14', false],
  //       constitution: ['16', false],
  //       intelligence: ['10', false],
  //       wisdom: ['16', false],
  //       charisma: ['10', false],
  //     },
  //     skills: {
  //       acrobatics: ['12', false],
  //       animalHandling: ['11', false],
  //       arcana: ['11', false],
  //       athletics: ['11', false],
  //       deception: ['11', false],
  //       history: ['12', false],
  //       insight: ['13', false],
  //       intimidation: ['14', false],
  //       investigation: ['16', false],
  //       medicine: ['15', false],
  //       nature: ['17', false],
  //       perception: ['18', false],
  //       performance: ['19', false],
  //       persuasion: ['20', false],
  //       religion: ['9', false],
  //       sleightOfHand: ['8', false],
  //       stealth: ['7', false],
  //       survival: ['6', false],
  //     },
  //     combatStats: {
  //       armorClass: '18',
  //       initiative: '+2',
  //       speed: '30',
  //       hitPointMaximum: '12',
  //       currentHitPoints: '12',
  //       temporaryHitPoints: '0',
  //       hitDice: '',
  //       deathSaveSuccesses: '0',
  //       deathSaveFailures: '0',
  //     },
  //     appearance: {
  //       age: '',
  //       height: '',
  //       weight: '',
  //       eyes: '',
  //       skin: '',
  //       hair: '',
  //     },
  //     personality: {
  //       personalityTraits: '',
  //       ideals: '',
  //       bonds: '',
  //       flaws: '',
  //     },
  //     weapons: {
  //       name: '',
  //       attackBonus: '',
  //       damage: '',
  //       damageType: '',
  //       range: '',
  //       description: '',
  //     },
  //   };
  //   this.character = char;
  //   // console.log('character: ', this.character);
  // }

  public character: any = {};

  // #region character data retrieval

  public getCompleteCharacter(): Character {
    return this.character;
  }

  public getCharacterData(): any {
    return this.character.characterData;
  }

  public getAppearance(): any {
    return this.character.appearance;
  }

  public getAttributes(): any {
    return this.character.attributes;
  }

  public getSkills(): any {
    return this.character.skills;
  }

  public getCombatStats(): any {
    return this.character.combatStats;
  }

  public getPersonality(): any {
    return this.character.personality;
  }

  // public getWeapons(): any {
  //   return this.character.weapons;
  // }

  // #endregion

  // #region data storage

  private weapons: Weapon[] = [
    {
      name: 'Großschwert',
      attackBonus: '+3',
      damage: [{ diceNumber: '2', diceType: 'd6', damageType: 'slashing' }],
      proficient: true,
      range: '5ft',
      throwRange: '',
      isFinesse: false,
      isVersatile: false,
      isTwoHanded: true,
      isRanged: false,
      canBeThrown: false,
      weight: 'schwer',
      isMagical: false,
    },
    {
      name: 'Kurzbogen +1',
      attackBonus: '+5',
      damage: [{ diceNumber: '1', diceType: 'd8', damageType: 'piercing' }],
      proficient: false,
      range: '80/320ft',
      throwRange: '',
      isFinesse: false,
      isVersatile: false,
      isTwoHanded: true,
      isRanged: true,
      canBeThrown: false,
      weight: 'leicht',
      isMagical: true,
      magicBonus: 1,
    },
    {
      name: 'Feuerstab',
      attackBonus: '+5',
      damage: [
        { diceNumber: '1', diceType: 'd6', damageType: 'bludgeoning' },
        { diceNumber: '1', diceType: 'd4', damageType: 'fire' },
      ],
      versatileDamage: 'd8',
      proficient: true,
      range: '5ft',
      throwRange: '',
      isFinesse: false,
      isVersatile: true,
      isTwoHanded: false,
      isRanged: false,
      canBeThrown: false,
      weight: 'normal',
      isMagical: false,
    },
  ];

  public getWeapons(): Weapon[] {
    return this.weapons;
  }

  public setWeapons(weapons: Weapon[]): void {
    this.weapons = weapons;
    console.log('weapons updated from data service: ', this.weapons);
  }

  private spells: Spell[] = [
    {
      name: 'Feuerball',
      level: 3,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: true,
      needsConcentration: false,
      needsAttackRoll: false,
      needsSavingThrow: true,
      savingThrowAttribute: 'dexterity',
      school: 'Evocation',
      doesDamage: true,
      damage: [
        { diceNumber: '8', diceType: 'd6', damageType: 'fire' },
        { diceNumber: '8', diceType: 'd6', damageType: 'fire' },
      ],
      isRanged: true,
      doesHeal: false,
      description:
        'Ein heller Strahl schießt von deinem Zeigefinger zu einem von dir gewählten Punkt in Reichweite und blüht dann mit einem tiefen Brüllen zu einer Flammenexplosion auf. Jede Kreatur in einem Umkreis von 6 Metern um diesen Punkt muss einen Rettungswurf auf Geschicklichkeit machen. Bei einem misslungenen Rettungswurf erleidet das Ziel 8W6 Feuerschaden, bei einem erfolgreichen Wurf die Hälfte des Schadens. Das Feuer breitet sich um Ecken aus. Es entzündet brennbare Gegenstände in der Umgebung, die nicht getragen werden. Wenn du diesen Zauber mit einem Zauberplatz des 4. Grades oder höher wirkst, erhöht sich der Schaden um 1W6 für jeden Grad über dem 3.',
      range: 150,
      hasAreaOfEffect: true,
      radius: 20,
      areaOfEffectType: 'sphere',
    },
    {
      name: 'Feuerpfeil',
      level: 0,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: true,
      needsConcentration: false,
      needsAttackRoll: true,
      needsSavingThrow: false,
      school: 'Evocation',
      doesDamage: true,
      doesHeal: false,
      description:
        'Du schleuderst einen Splitter aus Feuer auf eine Kreatur in Reichweite. Lege einen Fernkampf-Zauberangriff gegen das Ziel ab. Bei einem Treffer erleidet das Ziel 1W10 Feuerschaden. Ein brennbarer Gegenstand, der von diesem Zauber getroffen wird, geht in Flammen auf, wenn er nicht getragen oder in der Hand gehalten wird. Der Schaden dieses Zaubers steigt um 1W10, wenn du die 5. Stufe (2W10), die 11. Stufe (3W10) und die 17. Stufe (4W10) erreichst.  ',
      damage: [{ diceNumber: '1', diceType: 'd10', damageType: 'fire' }],
      isRanged: true,
      range: 120,
      hasAreaOfEffect: false,
    },
    {
      name: 'Vampiric Touch',
      level: 3,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: true,
      needsAttackRoll: true,
      needsSavingThrow: false,
      doesDamage: true,
      doesHeal: false,
      description:
        'Die Berührung deiner schattenumrankten Hand kann anderen die Lebenskraft entziehen, um deine Wunden zu heilen. Führe einen Nahkampf-Zauberangriff gegen eine Kreatur in deiner Reichweite aus. Bei einem Treffer erleidet das Ziel 3W6 nekrotischen Schaden, und du erhältst Trefferpunkte in Höhe der Hälfte des zugefügten nekrotischen Schadens zurück. Bis der Zauber endet, kannst du den Angriff in jeder deiner Runden als Aktion wiederholen. Wenn du diesen Zauber mit einem Zauberplatz des 4. Grades oder höher wirkst, erhöht sich der Schaden um 1W6 für jeden Grad über dem 3.',
      school: 'Necromancy',
      damage: [{ diceNumber: '3', diceType: 'd6', damageType: 'necrotic' }],
      isRanged: false,
      range: 5,
      hasAreaOfEffect: false,
    },
    {
      name: 'Heilende Berührung',
      level: 1,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: false,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      doesHeal: true,
      healAmount: { diceNumber: '1', diceType: 'd8', additionalHeal: 5 },
      description:
        'Deine Berührung kann Wunden heilen. Berühre eine Kreatur und spende ihr 1W8 + deinem Fertigkeitsbonus Trefferpunkte. Dieser Zauber hat keine Wirkung auf Untote oder Konstrukte. Wenn du diesen Zauber mit einem Zauberplatz des 2. Grades oder höher wirkst, erhöht sich die Heilung um 1W8 für jeden Grad über dem 1.',
      school: 'Evocation',
      isRanged: false,
      range: 5,
      hasAreaOfEffect: false,
    },
  ];

  public getSpells(): Spell[] {
    return this.spells;
  }

  public setSpells(spells: Spell[]): void {
    this.spells = spells;
    console.log('spells updated from data service: ', this.spells);
  }

  private abilities: Ability[] = [
    {
      name: 'Feenschritt',
      charges: 8,
      currentlyUsedCharges: 4,
      cost: 'bonus',
      shortDescription:
        'Als Bonusaktion kannst du dich bis zu 30 Fuß zu einem Punkt teleportieren, den du sehen kannst.',
      longDescription:
        'Als Bonusaktion kannst du dich auf magische Weise bis zu 30 Fuß weit in einen unbesetzten Raum teleportieren, den du sehen kannst. Du kannst diese Eigenschaft so oft einsetzen, wie es deinem Fertigkeitsbonus entspricht, und du erhältst alle verbrauchten Einsätze zurück, wenn du eine lange Pause machst.',
    },
    {
      name: 'Durchschnaufen',
      charges: 1,
      currentlyUsedCharges: 0,
      cost: 'bonus',
      shortDescription:
        'Als Bonusaktion kannst du dich um 1 d10 + Fighter Level Trefferpunkten heilen.',
      longDescription:
        'Als Bonusaktion kannst du dich um 1 d10 + Fighter Level Trefferpunkten heilen. Du kannst diese Fähigkeit einmal pro langer Rast einsetzen.',
    },
    {
      name: 'Tatendrang',
      charges: 1,
      currentlyUsedCharges: 1,
      cost: 'none',
      shortDescription: 'Du kannst eine weitere Aktion ausführen.',
      longDescription:
        'Du kannst eine weitere Aktion ausführen. Du kannst diese Fähigkeit einmal pro langer Rast einsetzen.',
    },
  ];

  public getAbilities(): Ability[] {
    return this.abilities;
  }

  public setSAbilities(abilities: Ability[]): void {
    this.abilities = abilities;
    console.log('abilities updated from data service: ', this.abilities);
  }

  private traits: Trait[] = [
    {
      name: 'Feenblut',
      shortDescription:
        'Du hast Vorteil auf alle Rettungswürfe gegen Zauber und andere magische Effekte.',
      longDescription:
        'Du hast Vorteil auf alle Rettungswürfe gegen Zauber und andere magische Effekte.',
      origin: 'Elf',
    },
    {
      name: 'Dunkelsicht',
      shortDescription: 'Du hast 60 ft. Dunkelsicht.',
      longDescription:
        'Du hast 60 ft. Dunkelsicht. Du kannst in völliger Dunkelheit so klar sehen wie am Tag.',
      origin: 'Elf',
    },
    {
      name: 'Fey Ancestry',
      shortDescription:
        'Du meditierst und musst nicht schlafen. Du kannst nicht durch Magie in Schlaf versetzt werden.',
      longDescription:
        'Du meditierst und musst nicht schlafen. Du kannst nicht durch Magie in Schlaf versetzt werden.',
      origin: 'Elf',
    },
  ];

  public getTraits(): Trait[] {
    return this.traits;
  }

  public setTraits(traits: Trait[]): void {
    this.traits = traits;
    console.log('traits updated from data service: ', this.traits);
  }

  private spellslots: any = {
    spellslots: [
      { totalSlots: 4, usedSlots: 2 },
      { totalSlots: 2, usedSlots: 1 },
    ],
    showSpellslots: true,
  };

  public getSpellslots(): any {
    return this.spellslots;
  }

  public setSpellslots(spellslots: any): void {
    this.spellslots = spellslots;
    console.log('spellslots updated from data service: ', this.spellslots);
  }

  private kiPoints: any = {
    totalPoints: 4,
    usedPoints: 2,
    showKiPoints: true,
  };

  public getKiPoints(): any {
    return this.kiPoints;
  }

  public setKiPoints(kiPoints: any): void {
    this.kiPoints = kiPoints;
    console.log('kiPoints updated from data service: ', this.kiPoints);
  }

  private proficiencies: any = {
    armor: {
      light: true,
      medium: true,
      heavy: false,
    },
    weapons: {
      simple: true,
      martial: true,
      other: ['Kurzschwert', 'Gleve'],
    },
    tools: ['Schmiedewerkzeuge', 'Würfel'],
    languages: ['Gemeinsprache', 'Elfisch', 'Zwergisch'],
  };

  public getProficiencies(): any {
    return this.proficiencies;
  }

  public setProficiencies(proficiencies: any): void {
    this.proficiencies = proficiencies;
    console.log(
      'proficiencies updated from data service: ',
      this.proficiencies
    );
  }

  // #region Character Data observables

  /**
   *
   * @param newValue The new value of the Character Data
   */
  public updateCharacterData(newValue: any) {
    const functionCall: string = newValue.name + 'Subject.next(newValue)';
    eval(`this.${functionCall}`);
    console.log('updateProperty: ', newValue.value);
    // TODO: Datenbank updaten
  }

  private proficiencySubject = new BehaviorSubject<any>({
    name: 'proficiencyBonus',
    value: 2,
  });
  public proficiency$ = this.proficiencySubject.asObservable();

  private _hitPoints = {
    maxHitPoints: 10,
    currentHitPoints: 8,
    temporaryHitPoints: 1,
  };

  public set hitPoints(hitPointsData: any) {
    console.log('hitPointsData: ', hitPointsData);
    this._hitPoints = hitPointsData;
  }

  public get hitPoints(): any {
    return this._hitPoints;
  }

  private _armorClass: number = 19;

  public get armorClass(): number {
    return this._armorClass;
  }

  public set armorClass(newValue: number) {
    console.log('armorClass: ', newValue);
    this._armorClass = newValue;
  }

  private _initiative: number = 2;

  public get initiative(): number {
    return this._initiative;
  }

  public set initiative(newValue: number) {
    console.log('initiative: ', newValue);
    this._initiative = newValue;
  }

  private _movement: number = 30;

  public get movement(): number {
    return this._movement;
  }

  public set movement(newValue: number) {
    console.log('movement: ', newValue);
    this._movement = newValue;
  }

  private _hitDice: any[] = [{ diceNumber: '1', diceType: 'd10' }];

  public get hitDice(): any[] {
    return this._hitDice;
  }

  public set hitDice(newValue: any[]) {
    console.log('hitDice: ', newValue);
    this._hitDice = newValue;
  }

  private _deathSaves: number[] = [0, 0];

  public get deathSaves(): number[] {
    return this._deathSaves;
  }

  public set deathSaves(newValue: number[]) {
    console.log('deathSaves: ', newValue);
    this._deathSaves = newValue;
  }

  private _inspiration: boolean = false;

  public get inspiration(): boolean {
    return this._inspiration;
  }

  public set inspiration(newValue: boolean) {
    console.log('inspiration: ', newValue);
    this._inspiration = newValue;
  }

  private _experience: number = 0;

  public get experience(): number {
    return this._experience;
  }

  public set experience(newValue: number) {
    console.log('experience: ', newValue);
    this._experience = newValue;
  }

  private _exhaustion: number = 0;

  public get exhaustion(): number {
    return this._exhaustion;
  }

  public set exhaustion(newValue: number) {
    console.log('exhaustion: ', newValue);
    this._exhaustion = newValue;
  }

  private _conditions: string[] = ['Blind', 'Kampfunfähig'];

  public get conditions(): string[] {
    return this._conditions;
  }

  public set conditions(newValue: string[]) {
    console.log('conditions: ', newValue);
    this._conditions = newValue;
  }

  // #endregion

  /**
   *
   * @param newValue The new value of the Attribute
   */
  updateAttribute(newValue: Attribute) {
    const functionCall: string = newValue.name + 'Subject.next(newValue)';
    eval(`this.${functionCall}`);
    console.log('updateAttribute: ', newValue.name);
    // TODO: Datenbank updaten
  }

  private strengthSubject = new BehaviorSubject<Attribute>(
    { name: 'strength', value: 19, proficiency: true } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public strength$ = this.strengthSubject.asObservable();

  private dexteritySubject = new BehaviorSubject<Attribute>(
    { name: 'dexterity', value: 17, proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public dexterity$ = this.dexteritySubject.asObservable();

  private constitutionSubject = new BehaviorSubject<Attribute>(
    { name: 'constitution', value: 12, proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public constitution$ = this.constitutionSubject.asObservable();

  private intelligenceSubject = new BehaviorSubject<Attribute>(
    { name: 'intelligence', value: 13, proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public intelligence$ = this.intelligenceSubject.asObservable();

  private wisdomSubject = new BehaviorSubject<Attribute>(
    { name: 'wisdom', value: 13, proficiency: true } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public wisdom$ = this.wisdomSubject.asObservable();

  private charismaSubject = new BehaviorSubject<Attribute>(
    { name: 'charisma', value: 19, proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public charisma$ = this.charismaSubject.asObservable();

  // #endregion

  // #region Skill observables

  /**
   *
   * @param newValue The new value of the skill
   */
  updateSkill(newValue: Skill) {
    const functionCall: string = newValue.name + 'Subject.next(newValue)';
    eval(`this.${functionCall}`);
    console.log('updateSkill: ', newValue.name);
    // TODO: Datenbank updaten
  }

  private acrobaticsSubject = new BehaviorSubject<Skill>(
    { name: 'acrobatics', proficiency: true } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public acrobatics$ = this.acrobaticsSubject.asObservable();

  private animalHandlingSubject = new BehaviorSubject<Skill>(
    { name: 'animalHandling', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public animalHandling$ = this.animalHandlingSubject.asObservable();

  private arcanaSubject = new BehaviorSubject<Skill>(
    { name: 'arcana', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public arcana$ = this.arcanaSubject.asObservable();

  private athleticsSubject = new BehaviorSubject<Skill>(
    { name: 'athletics', proficiency: true } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public athletics$ = this.athleticsSubject.asObservable();

  private deceptionSubject = new BehaviorSubject<Skill>(
    { name: 'deception', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public deception$ = this.deceptionSubject.asObservable();

  private historySubject = new BehaviorSubject<Skill>(
    { name: 'history', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public history$ = this.historySubject.asObservable();

  private insightSubject = new BehaviorSubject<Skill>(
    { name: 'insight', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public insight$ = this.insightSubject.asObservable();

  private intimidationSubject = new BehaviorSubject<Skill>(
    { name: 'intimidation', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public intimidation$ = this.intimidationSubject.asObservable();

  private investigationSubject = new BehaviorSubject<Skill>(
    { name: 'investigation', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public investigation$ = this.investigationSubject.asObservable();

  private medicineSubject = new BehaviorSubject<Skill>(
    { name: 'medicine', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public medicine$ = this.medicineSubject.asObservable();

  private natureSubject = new BehaviorSubject<Skill>(
    { name: 'nature', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public nature$ = this.natureSubject.asObservable();

  private perceptionSubject = new BehaviorSubject<Skill>(
    { name: 'perception', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public perception$ = this.perceptionSubject.asObservable();

  private performanceSubject = new BehaviorSubject<Skill>(
    { name: 'performance', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public performance$ = this.performanceSubject.asObservable();

  private persuasionSubject = new BehaviorSubject<Skill>(
    { name: 'persuasion', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public persuasion$ = this.persuasionSubject.asObservable();

  private religionSubject = new BehaviorSubject<Skill>(
    { name: 'religion', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public religion$ = this.religionSubject.asObservable();

  private sleightOfHandSubject = new BehaviorSubject<Skill>(
    { name: 'sleightOfHand', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public sleightOfHand$ = this.sleightOfHandSubject.asObservable();

  private stealthSubject = new BehaviorSubject<Skill>(
    { name: 'stealth', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public stealth$ = this.stealthSubject.asObservable();

  private survivalSubject = new BehaviorSubject<Skill>(
    { name: 'survival', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public survival$ = this.survivalSubject.asObservable();

  // #endregion

  // #region inventory

  private _items: SimpleItem[] = [
    {
      name: 'Kurzbogen +1',
      weight: 5,
      value: 50,
      quantity: 1,
      description: 'Hallo',
    },
    {
      name: 'Helebade',
      weight: 10,
      value: 5,
      quantity: 1,
      description: 'Hallo',
    },
    {
      name: 'Armbrust',
      weight: 8,
      value: 6,
      quantity: 1,
      description: 'Hallo',
    },
  ];

  public get items(): SimpleItem[] {
    return this._items;
  }

  public set items(newValue: SimpleItem[]) {
    console.log('items: ', newValue);
    this._items = newValue;
  }

  private _food: Food[] = [
    {
      name: 'Brot',
      quantity: 7,
      ready: true,
      weight: 0.5,
    },
    {
      name: 'Suppe',
      quantity: 4,
      ready: true,
      weight: 0.5,
    },
  ];

  public get food(): Food[] {
    return this._food;
  }

  public set food(newValue: Food[]) {
    console.log('food: ', newValue);
    this._food = newValue;
  }

  private _money: any = {
    copper: 100,
    silver: 100,
    electrum: 100,
    gold: 100,
    platinum: 100,
  };

  public get money(): any {
    return this._money;
  }

  public set money(newValue: any) {
    console.log('money: ', newValue);
    this._money = newValue;
  }

  private _consumbales: SimpleItem[] = [
    {
      name: 'Heiltrank',
      weight: 0.5,
      value: 50,
      quantity: 1,
      description: 'Heilt 1W8 + 2 Trefferpunkte',
    },
    {
      name: 'Öl',
      weight: 1,
      value: 1,
      quantity: 1,
      description: 'Brennt',
    },
  ];

  public get consumables(): SimpleItem[] {
    return this._consumbales;
  }

  public set consumables(newValue: SimpleItem[]) {
    console.log('consumables: ', newValue);
    this._consumbales = newValue;
  }

  private _miscellaneous: SimpleItem[] = [
    {
      name: 'Rucksack',
      weight: 5,
      value: 2,
      quantity: 1,
      description: 'Hallo',
    },
    {
      name: 'Seil',
      weight: 5,
      value: 2,
      quantity: 1,
      description: 'Hallo',
    },
  ];

  public get miscellaneous(): SimpleItem[] {
    return this._miscellaneous;
  }

  public set miscellaneous(newValue: SimpleItem[]) {
    console.log('misc: ', newValue);
    this._miscellaneous = newValue;
  }

  // #endregion

  // #region database calls

  public async addData(
    collection: string,
    data: any,
    key?: string
  ): Promise<void> {
    if (key) {
      return this.db
        .collection(collection)
        .add(data, key)
        .then(() => {});
    } else {
      return this.db
        .collection(collection)
        .add(data)
        .then(() => {});
    }
  }

  public getData(collection: string, key: string) {
    return this.db.collection(collection).doc(key).get();
  }

  public getCollection(collection: string) {
    return this.db.collection(collection).get();
  }

  public getCollectionWithKeys(collection: string) {
    return this.db.collection(collection).get({ keys: true });
  }

  public deleteCollection(collection: string) {
    this.db.collection(collection).delete();
  }

  // #endregion

  // #region character selection and construction

  /**
   *
   * @param character The name of the character that was selected
   */
  public async selectCharacter(character: string) {
    console.log('selectCharacter ausgewählt');
    const characterData = await this.getCollectionWithKeys(character);
    this.character = this.buildCurrentCharacter(characterData);
    console.log('character: ', this.character);
    this.Router.navigate(['journal']);
  }

  private buildCurrentCharacter(currentCharacterData: any): Character {
    const [
      appearanceData,
      attributesData,
      characterData,
      combatStatsData,
      personalityData,
      skillsData,
      weaponsData,
    ] = currentCharacterData.map((entry: any) => entry.data);

    const appearance = {
      age: appearanceData.age,
      height: appearanceData.height,
      weight: appearanceData.weight,
      eyes: appearanceData.eyes,
      skin: appearanceData.skin,
      hair: appearanceData.hair,
    };

    const attributes = {
      strength: attributesData.strength,
      dexterity: attributesData.dexterity,
      constitution: attributesData.constitution,
      intelligence: attributesData.intelligence,
      wisdom: attributesData.wisdom,
      charisma: attributesData.charisma,
    };

    const character = {
      name: characterData.name,
      race: characterData.race,
      class: characterData.class,
      subclass: characterData.subclass,
      level: characterData.level,
      background: characterData.background,
      experience: characterData.experience,
      inspiration: characterData.inspiration,
      proficiencyBonus: characterData.proficiencyBonus,
    };

    const combatStats = {
      armorClass: combatStatsData.armorClass,
      initiative: combatStatsData.initiative,
      speed: combatStatsData.speed,
      hitPointMaximum: combatStatsData.hitPointMaximum,
      currentHitPoints: combatStatsData.currentHitPoints,
      temporaryHitPoints: combatStatsData.temporaryHitPoints,
      hitDice: combatStatsData.hitDice,
      deathSaveSuccesses: combatStatsData.deathSaveSuccesses,
      deathSaveFailures: combatStatsData.deathSaveFailures,
    };

    const personality = {
      personalityTraits: personalityData.personalityTraits,
      ideals: personalityData.ideals,
      bonds: personalityData.bonds,
      flaws: personalityData.flaws,
    };

    const skills = {
      acrobatics: skillsData.acrobatics,
      animalHandling: skillsData.animalHandling,
      arcana: skillsData.arcana,
      athletics: skillsData.athletics,
      deception: skillsData.deception,
      history: skillsData.history,
      insight: skillsData.insight,
      intimidation: skillsData.intimidation,
      investigation: skillsData.investigation,
      medicine: skillsData.medicine,
      nature: skillsData.nature,
      perception: skillsData.perception,
      performance: skillsData.performance,
      persuasion: skillsData.persuasion,
      religion: skillsData.religion,
      sleightOfHand: skillsData.sleightOfHand,
      stealth: skillsData.stealth,
      survival: skillsData.survival,
    };
    const weapons = {
      name: weaponsData.name,
      attackBonus: weaponsData.attackBonus,
      damage: weaponsData.damage,
      damageType: weaponsData.damageType,
      range: weaponsData.range,
      description: weaponsData.damageDice,
    };
    const newCharacter: Character = {
      appearance: appearance,
      attributes: attributes,
      characterData: character,
      combatStats: combatStats,
      personality: personality,
      skills: skills,
      weapons: weapons,
    };
    console.log('newCharacter: ', newCharacter);
    return newCharacter;
  }
  // #endregion
}
